// Craig S. Kaplan

// A more complicated visualization of the information in the
// Public Sector Salary Disclosure -- collect employees in the
// University Sector, group them by institution, and order the
// institutions according to their median salaries.

// This is more than I'd expect a CS 106 student to write, but
// it's an interesting visualization and worth sharing.

float stringToSalary( String sal )
{
  String digs = join( splitTokens( sal, "$,.-" ), "" );
  if ( digs.length() > 0 ) {
    return float(digs) / 100.0;
  } else {
    return 0;
  }
}

// This median calculation assumes going in that the records
// are already sorted by salary.

float getMedianSalary( Table table, String name )
{
  float[] sals = {};
  for ( int idx = 0; idx < table.getRowCount(); ++idx ) {
    if ( table.getString( idx, "Employer" ).equals( name ) ) {
      sals = append( sals, table.getFloat( idx, "TotalSalary" ) );
    }
  }

  if ( sals.length % 2 == 0 ) {
    return 0.5*(sals[sals.length/2-1]+sals[sals.length/2]);
  } else {
    return sals[sals.length/2];
  }
}

void setup()
{
  size( 800, 600 );
  background( 255 );
  noStroke();

  // Load in the CSV.
  Table table = loadTable( "pssd.csv", "header" );
  table.addColumn( "TotalSalary" );

  for ( int idx = 0; idx < table.getRowCount(); ++idx ) {
    float a = stringToSalary( table.getString( idx, "Salary Paid" ) );
    float b = stringToSalary( table.getString( idx, "Taxable Benefits" ) );

    table.setFloat( idx, "TotalSalary", a + b );
  }
  table.sortReverse( "TotalSalary" );

  // Compute the maximum salary in the table.
  float maxsal = 0;
  for ( int idx = 0; idx < table.getRowCount(); ++idx ) {
    if ( table.getString( idx, "Sector" ).equals( "Universities" ) ) {
      float sal = table.getFloat( idx, "TotalSalary" );
      maxsal = max( sal, maxsal );
    }
  }
  // Scale it so that we know nothing will sit right at the right
  // edge of the sketch.
  maxsal = 1.05 * maxsal;

  FloatDict ms = new FloatDict();
  for ( int idx = 0; idx < table.getRowCount(); ++idx ) {
    if ( table.getString( idx, "Sector" ).equals( "Universities" ) ) {
      String name = table.getString( idx, "Employer" );
      if ( !ms.hasKey( name ) ) {
        float med = getMedianSalary( table, name );
        ms.set( name, med );
        // println( name, " => ", med );
      }
    }
  }
  ms.sortValuesReverse();
  IntDict mapper = new IntDict();
  int ct = 0;
  for ( String k : ms.keys() ) {
    mapper.set( k, ct );
    ++ct;
  }
  int num_inst = mapper.size();

  // Walk over every row in the table.
  for ( int idx = 0; idx < table.getRowCount(); ++idx ) {
    // Get the position string, and check whether it's likely to
    // belong to one of the categories we care about.  If so,
    // put it in that category.  Otherwise, may as well stick it
    // into "other".
    String pos = table.getString( idx, "Job Title" ).toLowerCase();
    String name = table.getString( idx, "Employer" );
    float sal = table.getFloat( idx, "TotalSalary" );
    
    if( !mapper.hasKey( name ) ) {
      continue;
    }

    // Find the X position where we're draw a marker for this
    // employee.
    float x = map( sal, 100000, maxsal, 200, width );
    float y = (mapper.get( name )+0.5) * float(height)/float(num_inst);

    if ( match( pos, "[aA]ssistant\\s+[pP]rofessor" ) != null ) {
      // Probably an "Assistant Professor".
      fill( #00ff00, 40 );
      ellipse( x, y, 10, 10 );
    } else if ( match( pos, "[aA]ssociate\\s+[pP]rofessor" ) != null ) {
      // Probably an "Associate Professor".
      fill( #0000ff, 40 );
      ellipse( x, y, 10, 10 );
    } else if ( match( pos, "[pP]rofessor" ) != null ) {
      // Professor, but neither assistant nor associate.
      // Probably a a "Full Professor" (the highest rank
      // for profs in North America).
      fill( #ff0000, 40 );
      ellipse( x, y, 10, 10 );
    } else {
      // Couldn't figure out what this person does (which
      // could happen for a number of reasons).  So put them
      // into the "other" category.
      fill( #404040, 40 );                  
      ellipse( x, y, 10, 10 );
    }
  }

  textSize( 10 );
  textAlign( RIGHT );
  float yy = 0;
  for ( String k : mapper.keys() ) {
    fill( 0, 255 );
    noStroke();
    text( k, 190, yy + 9 );
    stroke( 255, 255 );
    strokeWeight( 2.0 );
    noFill();
    float m = ms.get( k );
    float x = map( m, 100000, maxsal, 200, width );
    line( x, yy, x, yy + float(height) / float(num_inst) );

    yy += float(height) / float(num_inst);
  }
  
  save( "viz.png" );
}